%{
 This program implements a Randles circuit model using classic parameter
 estimation and a non-linear system identification method, respectively.

 To start the program, run this script with no arguments. That will
 generate an output struct with the model parameters and graphs comparing
 the model performance with the experimental data.

%}
function out = mainCircuit()
%% Filepaths
folderFile = mfilename('fullpath');
file = mfilename();
folder = folderFile(1:end-length(file));
addpath(genpath(folder))

close all
%% Available data sets
input = load('input.mat');
input = input.input;
voltages = [1.1 0.9 0.7 0.5 0.9];
voltNames = ["V1_1" "V0_9" "V0_7" "V0_5"];

% Choose data set index:
voltID = 1;

%% Load Data
yData = input.(voltNames(voltID)).yData;
uData = input.(voltNames(voltID)).uData; 

%% Parameters
% Classic parameter extraction (function getPars) is made to either get the
% system parameters directly and predict the final results, or to obtain a
% paramter initialization for the system identification model.

% To extract model parameters, the below timestamps indicate the data
% sampling time, at what time saturation is reached in the first cycle, and
% when the first cycle ends (if tSample is changed the getPars function 
% should be changed too)
tSample = 60;      % Unit s
t_saturation = 50; % Unit min
t_endCycle = 200;  % Unit min

for predict = [true,false] %Classic (true) or system identification (false)
    [R1inv,R2inv,Cinv] = getPars(yData,t_saturation,...
                                 t_endCycle,tSample,voltages(voltID));
    parameters = {R1inv;R2inv;Cinv};
    
    %% Create model           
    data = iddata(yData,uData,tSample,'Name','Salt');
    order = [1 1 3];                   % Number of states
    Ts = 0;                            % Model type: 0 -> continous
    initial_states =  [0;0;0];                                                              
    fileName = 'greyFun';              % File defining diff-eq.
    model = idnlgrey(fileName, order, parameters, initial_states, Ts,...
                     'Name','Salt');         

    if predict
        setpar(model,'Fixed', {true,true,true}); 
    end

    opt = nlgreyestOptions('Display', 'Full');  % Show progress
    opt.Regularization.Lambda = 0;              % Regularization
    opt.SearchOptions.Advanced.MaxIter=100;     % Iteration steps

    %% Compute

    FinalModel = nlgreyest(data,model,opt);     % Compute results

    out.R1inv = FinalModel.Parameters(1).Value; 
    out.R2inv = FinalModel.Parameters(2).Value; 
    out.Cinv  = FinalModel.Parameters(3).Value;
    [y,~,~] = compare(data,FinalModel);         % Read simulated data


    %% Plot
    Tmin = 1; Tmax = size(yData,1);             % Plot range                           
    figure;
    plot((Tmin:Tmax)*tSample/60,yData(Tmin:Tmax),...
         (Tmin:Tmax)*tSample/60,y.OutputData(Tmin:Tmax),'LineWidth',2);
    xlabel('Time (min)')                      
    ylabel('Passed current (C)')     
    if predict
        legend('Data','Classic model fit')
    else
        legend('Data','Model fit')
    end
end
%% Classic parameter extraction
%{
 This function calculates the R1, R2 and C parameters of the Randles
 circuit model directly from the graphs.

 R1 is calculated from the intital slope (V = R*I).
 C is calculated from from the charge release:
%}
    function [R1_inv,R2_inv,C_inv] = getPars(data,t_sat,t_end,t_sample,V)
        bufferFactor = 4;
        yCycle = data(1:t_end);
        [~,t_switch] = max(yCycle);
        % R1 = V/I_start
        R1 = V/((yCycle(round(t_sat/bufferFactor))-yCycle(1))/t_sample/round(t_sat/bufferFactor));
        % I_sat*(R1+R2)=V
        R2 = -R1 + V/((yCycle(t_switch)-yCycle(t_sat))/t_sample/(t_switch-t_sat));
        % C = q_sat/V
        C = (yCycle(t_switch)-yCycle(t_end))/V;
        
        R1_inv = 1/R1; R2_inv = 1/R2; C_inv = 1/C;
    end
end



